<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Actions\DeleteEvent;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Form\Facades\FormSubmission;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\DefaultFields;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Inertia\Inertia;
use Inertia\Response;

class EventController extends Controller
{
    public function index(): Response
    {
        $table = Table::make(Event::class)
            ->columns(request('cols'))
            ->size(request('size'))
            ->page(request('page'))
            ->search(request('query'))
            ->filters(request('filter'))
            ->get();

        return Inertia::render('Event/Index', [
            'data' => $table,
        ]);
    }

    public function create(): Response
    {
        $fields = DefaultFields::forCreation(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class)
            ->addAction(static function () {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.index');
            })
            ->addAction(static function () {
                return Button::make('Create Event')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.store');
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add('Create an event')
            ->get();

        return Inertia::render('Event/Create', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
        ]);
    }

    public function store(): RedirectResponse
    {
        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        FormSubmission::store($form, request()->input());

        return redirect()->route('admin.event.index');
    }

    public function edit(Event $event): Response
    {
        $form = Form::fromModel($event)
            ->addAction(static function () {
                return Button::make('Cancel')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.index');
            })
            ->addAction(static function () use ($event) {
                return Button::make('Update Event')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.update', ['event' => $event]);
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($form->getValue('name'), route('admin.event.show', $event->id))
            ->add('Edit event')
            ->get();

        return Inertia::render('Event/Edit', [
            'form' => $form,
            'breadcrumbs' => $breadcrumbs,
            'eventName' => $form->getValue('name'),
        ]);
    }

    public function update(Event $event): RedirectResponse
    {
        $form = Form::fromModel($event);

        FormSubmission::update($form, request()->input());

        return redirect()->route('admin.event.index');
    }

    public function show(Event $event): Response
    {
        $form = Form::display($event)
            ->addAction(static function () use ($event) {
                return Button::make('Delete')
                    ->variant('secondary')
                    ->method(InertiaMethod::POST())
                    ->route('admin.event.delete', ['event' => $event]);
            })
            ->addAction(static function () use ($event) {
                return Button::make('Edit')
                    ->variant('secondary')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.edit', ['event' => $event->id]);
            });

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($form->getValue('name'))
            ->get();

        return Inertia::render('Event/Show', [
            'data' => $form,
            'breadcrumbs' => $breadcrumbs,
            'eventName' => $form->getValue('name'),
        ]);
    }

    public function delete(Event $event, DeleteEvent $deleteEvent): RedirectResponse
    {
        $deleteEvent->execute($event);

        return redirect()->route('admin.event.index');
    }
}
