<?php

namespace EventPortal\Tests\Unit\Form;

use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Form\Facades\FormSubmission;
use EventPortal\Domain\Response\Models\Response;
use EventPortal\Domain\Util\DefaultFields;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Validation\ValidationException;

class FormSubmissionTest extends TestCase
{
    use DatabaseMigrations;

    /** @test **/
    public function can_store_form()
    {
        $default = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                    ],
                ],
            ]
        ];

        config()->set('defaults.event.fields', $default);

        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        FormSubmission::store($form, ['name' => 'Hello World!']);

        $field = Field::firstWhere('name', 'name');

        $this->assertFieldHasResponse($field, 'Hello World!');
    }

    /** @test **/
    public function can_update_form()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $field = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $field->id,
                'value' => 'Hello World!',
            ]);

        $form = Form::fromModel($event);

        FormSubmission::update($form, ['name' => 'Goodbye World!']);

        $field = Field::firstWhere('name', 'name');

        $this->assertFieldHasResponse($field, 'Goodbye World!');
    }

    /** @test **/
    public function validation_is_thrown_on_store()
    {
        $default = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'label' => 'Event name',
                        'name' => 'name',
                        'component' => FieldType::TextInput()->getValue(),
                        'validation' => ['required'],
                    ],
                ],
            ]
        ];

        config()->set('defaults.event.fields', $default);

        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        try {
            FormSubmission::store($form, []);
        } catch (ValidationException $exception) {
            $this->assertArrayHasKey('name', $exception->errors());
        }
    }

    /** @test **/
    public function validation_is_thrown_on_update()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)
            ->states('Section')
            ->create([
                'label' => 'Key details',
                'model_id' => $event->id,
                'model_type' => get_class($event),
            ]);

        $field = factory(Field::class)
            ->states('TextInput')
            ->create([
                'label' => 'Event name',
                'name' => 'name',
                'model_id' => $event->id,
                'model_type' => get_class($event),
                'section_id' => $section->id,
                'data' => ['validation' => ['required']],
            ]);

        factory(Response::class)
            ->create([
                'field_id' => $field->id,
                'value' => 'Hello World!',
            ]);

        $form = Form::fromModel($event);

        try {
            FormSubmission::update($form, []);
        } catch (ValidationException $exception) {
            $this->assertArrayHasKey('name', $exception->errors());
        }
    }

}
