<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Field\Enums\FieldType;
use EventPortal\Domain\Field\Models\Field;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\DefaultFields;
use EventPortal\Domain\Util\ValidationRules;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class ValidationRulesTest extends TestCase
{
    use DatabaseMigrations;

    /** @test **/
    public function can_get_validation_rules_for_form_with_model()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)->states('Section')->create(['model_id' => $event->id]);

        factory(Field::class)->create([
            'name' => 'event_name',
            'section_id' => $section->id,
            'model_id' => $event->id,
            'data' => ['validation' => ['required']],
        ]);

        $form = Form::fromModel($event);

        $actual = ValidationRules::forForm($form, 'update');

        $this->assertEquals(['event_name' => ['required']], $actual);
    }

    /** @test */
    public function can_get_validation_rules_for_form_from_config()
    {
        $config = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'component' => FieldType::TextInput()->getValue(),
                        'name' => 'event_name',
                        'label' => 'Event name',
                        'classes' => ['col-6'],
                        'validation' => [
                            'required',
                        ],
                        'placeholder' => 'E.g. Congress meeting',
                    ],
                ],
            ],
        ];

        config()->set('defaults.event.fields', $config);

        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        $actual = ValidationRules::forForm($form, 'create');

        $this->assertEquals(['event_name' => ['required']], $actual);
    }

    /** @test **/
    public function can_get_update_validation_for_form_from_config()
    {
        $config = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'component' => FieldType::TextInput()->getValue(),
                        'name' => 'event_name',
                        'label' => 'Event name',
                        'classes' => ['col-6'],
                        'validation' => [
                            'required',
                        ],
                        'updateValidation' => [
                            'email',
                        ],
                        'placeholder' => 'E.g. Congress meeting',
                    ],
                ],
            ],
        ];

        config()->set('defaults.event.fields', $config);

        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        $actual = ValidationRules::forForm($form, 'update');

        $this->assertEquals(['event_name' => ['required', 'email']], $actual);
    }

    /** @test **/
    public function can_get_create_validation_for_form_from_config()
    {
        $config = [
            [
                'label' => 'Key details',
                'component' => FieldType::Section()->getValue(),
                'children' => [
                    [
                        'component' => FieldType::TextInput()->getValue(),
                        'name' => 'event_name',
                        'label' => 'Event name',
                        'classes' => ['col-6'],
                        'validation' => [
                            'required',
                        ],
                        'updateValidation' => [
                            'email',
                        ],
                        'createValidation' => [
                            'min:2',
                        ],
                        'placeholder' => 'E.g. Congress meeting',
                    ],
                ],
            ],
        ];

        config()->set('defaults.event.fields', $config);

        $fields = DefaultFields::forStoring(Event::class);

        $form = Form::fromDefaultFields($fields, Event::class);

        $actual = ValidationRules::forForm($form, 'create');

        $this->assertEquals(['event_name' => ['required', 'min:2']], $actual);
    }


    /** @test **/
    public function can_get_update_validation_for_form_from_model()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)->states('Section')->create(['model_id' => $event->id]);

        factory(Field::class)->create([
            'name' => 'event_name',
            'section_id' => $section->id,
            'model_id' => $event->id,
            'data' => ['validation' => ['required'], 'updateValidation' => ['min:2'], 'createValidation' => ['email']],
        ]);

        $form = Form::fromModel($event);

        $actual = ValidationRules::forForm($form, 'update');

        $this->assertEquals(['event_name' => ['required', 'min:2']], $actual);
    }

    /** @test **/
    public function can_get_create_validation_for_form_from_model()
    {
        $event = factory(Event::class)->create();

        $section = factory(Field::class)->states('Section')->create(['model_id' => $event->id]);

        factory(Field::class)->create([
            'name' => 'event_name',
            'section_id' => $section->id,
            'model_id' => $event->id,
            'data' => ['validation' => ['required'], 'updateValidation' => ['min:2'], 'createValidation' => ['email']],
        ]);

        $form = Form::fromModel($event);

        $actual = ValidationRules::forForm($form, 'create');

        $this->assertEquals(['event_name' => ['required', 'email']], $actual);
    }
}
